var ipcMain = Electron.ipcMain;
import { NEST_RPC_INVOKE_EVENT } from "./constants";
const cache = {};
export class NestRPC {
    static register(ctx) {
        ipcMain.on(NEST_RPC_INVOKE_EVENT, async (e, cls, method, evt, ...params) => {
            const instance = cache[cls] || ctx.get(cls);
            if (!cache[cls]) {
                cache[cls] = instance;
            }
            NestRPC.handleParams(params, e, evt.callbacks);
            const data = await instance[method].apply(instance, ...params);
            e.sender.send(evt.event, data);
        });
    }
    static handleParams(params, e, callbacks) {
        const callbackEvents = [];
        const parameters = params.map((param, index) => {
            if (param.type === 'function') {
                const evt = callbacks.filter(cb => cb.index === index)[0];
                if (evt) {
                    return (...data) => e.sender.send(evt.event, ...data);
                }
                else {
                    return (...data) => void 0;
                }
            }
            else if (param.type === 'object' || param.type === 'boolean') {
                return JSON.parse(param.value);
            }
            else {
                return param.value;
            }
        });
        return { callbackEvents, parameters };
    }
}
//# sourceMappingURL=main.js.map